package net.esper.example.rfid;

import junit.framework.TestCase;
import net.esper.client.EPServiceProvider;
import net.esper.client.Configuration;
import net.esper.client.EPServiceProviderManager;
import net.esper.client.EPStatement;
import net.esper.client.time.TimerControlEvent;
import net.esper.example.support.SupportUpdateListener;

import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestStatementPerformance extends TestCase
{
    private static final Log log = LogFactory.getLog(TestStatementPerformance.class);
    private EPServiceProvider epService;

    public void setUp()
    {
        Configuration config = new Configuration();
        config.addEventTypeAlias("LocationReport", LocationReport.class);

        epService = EPServiceProviderManager.getDefaultProvider(config);
        epService.initialize();
        epService.getEPRuntime().sendEvent(new TimerControlEvent(TimerControlEvent.ClockType.CLOCK_EXTERNAL));
    }

    public void testPerformance()
    {
        //tryPerf(10000, 100000);
    }

    private void tryPerf(int numAssetGroups, int numEvents)
    {
        SupportUpdateListener listeners[] = new SupportUpdateListener[numAssetGroups];

        // Create statements
        log.debug(".tryPerf Creating statements for " + numAssetGroups + " groups");
        String[][] assetIds = new String[numAssetGroups][3];
        for (int i = 0; i < numAssetGroups; i++)
        {
            // Generate unique asset id over all groups
            assetIds[i][0] = String.format("%08d.1", i);   // 8 digit padded, i.e. 00000001.n;
            assetIds[i][1] = String.format("%08d.1", i);   // 8 digit padded, i.e. 00000002.n;
            assetIds[i][2] = String.format("%08d.1", i);   // 8 digit padded, i.e. 00000003.n;
            String assetIdList = "'" + assetIds[i][0] + "','" + assetIds[i][1] + "','" + assetIds[i][2] + "'";
            String streamName = "CountZone_" + i;

            String textOne = "insert into " + streamName +
                    " select zone, count(*) as cnt " +
                    "from LocationReport.std:unique('assetId') " +
                    "where assetId in (" + assetIdList + ") " +
                    "group by zone";
            epService.getEPAdministrator().createEQL(textOne);

            String textTwo = "select Part.zone from pattern [" +
                    "  every Part=" + streamName + "(cnt in [1:2]) ->" +
                    "  (timer:interval(60 sec) and not " + streamName + "(cnt in (0, 3)))]";
            EPStatement stmtTwo = epService.getEPAdministrator().createEQL(textTwo);
            listeners[i] = new SupportUpdateListener();
            stmtTwo.addListener(listeners[i]);
        }

        log.debug(".tryPerf Sending events");
        long start = System.currentTimeMillis();
        Random random = new Random();

        for (int i = 0; i < numEvents; i++)
        {
            if (i % 10000 == 0)
            {
                long now = System.currentTimeMillis();
                double deltaSec = (now - start) / 1000.0;
                double throughput = 10000.0 / deltaSec;
                System.out.println("total=" + i + " deltaSec=" + deltaSec + " per sec:" + throughput);
                for (int j = 0; j < listeners.length; j++)
                {
                    listeners[j].reset();
                }
                start = now;
            }

            int groupNum = random.nextInt() % numAssetGroups;
            int idNum = random.nextInt() % assetIds[0].length;
            int zone = random.nextInt() % 4;
            LocationReport report = new LocationReport(assetIds[groupNum][idNum], zone);
            epService.getEPRuntime().sendEvent(report);
        }
        long end = System.currentTimeMillis();
        long delta = end - start;
        System.out.println("total=" + numEvents + " delta=" + delta + " per sec:" + 10000.0 / (delta / 1000.0));
    }
}
